@@ -15,6 +15,7 @@
   :copyright: (c) 2013 by Robin Andeer
   :license: MIT, see LICENSE for more details
 """
+from datetime import datetime
 import sqlalchemy as sa
 from sqlalchemy.orm import relationship, backref
 
@@ -43,7 +44,8 @@ class GeneData(Base):
   completeness = sa.Column(sa.Float)
 
   # These column maps coverage/completeness to an individual+group
-  sample_id = sa.Column(sa.String)
+  sample_id = sa.Column(sa.String, sa.ForeignKey("Sample.id"))
+  sample = relationship("Sample", backref=backref("genes"))
   group_id = sa.Column(sa.Integer)
 
   # Genetic relationship
@@ -80,7 +82,8 @@ class TranscriptData(Base):
   completeness = sa.Column(sa.Float)
 
   # These column maps coverage/completeness to an individual+group
-  sample_id = sa.Column(sa.String)
+  sample_id = sa.Column(sa.String, sa.ForeignKey("Sample.id"))
+  sample = relationship("Sample", backref=backref("transcripts"))
   group_id = sa.Column(sa.Integer)
 
   # Genetic relationship
@@ -117,7 +120,8 @@ class ExonData(Base):
   completeness = sa.Column(sa.Float)
 
   # These column maps coverage/completeness to an individual+group
-  sample_id = sa.Column(sa.String)
+  sample_id = sa.Column(sa.String, sa.ForeignKey("Sample.id"))
+  sample = relationship("Sample", backref=backref("exons"))
   group_id = sa.Column(sa.Integer)
 
   # Genetic relationship
@@ -134,6 +138,40 @@ class ExonData(Base):
     self.coverage = coverage
     self.completeness = completeness
 
+class Sample(Base):
+  """
+  Stores meta-data about each sample. This helps out in consolidating all
+  important information in one place.
+
+  .. versionadded:: 0.4.0
+
+  :param str sample: The unique sample ID
+  :param str group: The unique group ID
+  :param int cutoff: The cutoff used for completeness
+  :param bool splice: ANS: Splice sites were included
+  :param str bam_file: Path to the BAM file used
+  """
+  __tablename__ = "Sample"
+
+  id = sa.Column(sa.String, primary_key=True)
+  group_id = sa.Column(sa.Integer)
+
+  cutoff = sa.Column(sa.Integer)
+  splice = sa.Column(sa.Boolean)
+  source = sa.Column(sa.String)
+  created_at = sa.Column(sa.DateTime, default=datetime.now)
+  updated_at = sa.Column(sa.DateTime, default=datetime.now,
+                         onupdate=datetime.now)
+
+  def __init__(self, sample, group, cutoff=10, source=None, splice=False):
+    super(Sample, self).__init__()
+
+    self.id = sample
+    self.group_id = group
+    self.cutoff = cutoff
+    self.source = source
+    self.splice = splice
+
 
 # =========================================================
 #   The actual Element Adapter
@@ -158,7 +196,8 @@ class ElementAdapter(ElementalDB):
     self.classes.update({
       "gene_data": GeneData,
       "transcript_data": TranscriptData,
-      "exon_data": ExonData
+      "exon_data": ExonData,
+      "sample": Sample
     })
 
   def transcriptStats(self, sample_id):
